#include <windows.h>
#include "Hw.h"

//-----------------------------------------------------------------------------------
// Functia PciBaseAddress
//-----------------------------------------------------------------------------------
// Functie:		- Cauta in memorie structura RSDP (Root System Description Pointer)
//				- Determina adresa tabelei RSDT (Root System Description Table)
//				- Citeste din tabela RSDT pointeri catre o tabela de descriere
//				  si cauta tabela ACPI MCFG corespunzatoare
//				- Citeste din tabela ACPI MCFG adresa de baza a spatiului de
//				  configuratie extins PCI Express
//-----------------------------------------------------------------------------------
// Parametri:	-
// Returneaza:	Adresa de baza a spatiului de configuratie extins PCI Express
//				0 - Nu s-a gasit tabela ACPI MCFG
//				1 - Nu s-a gasit structura RSDP
//-----------------------------------------------------------------------------------

DWORD64 PciBaseAddress(void)
{
	int i, j;
	bool bMcfg;
	DWORD dwAdr;
	DWORD dwMemVal;
	PBYTE pbAdrRSDT = NULL;
	PBYTE pbAdrTab  = NULL;
	PBYTE pbAdrMCFG = NULL;
	DWORD dwLungRSDT;
	DWORD dwAdrExtLo;
	DWORD dwAdrExtHi;
	DWORD64 qwAdrExt = 0;

	// Scaneaza memoria pentru cautarea semnaturii "RSD PTR "
	for (dwAdr = 0x000E0000; dwAdr < 0x000FFFF0; dwAdr += 0x10)  {
		dwMemVal = _inmdw((DWORD_PTR)dwAdr);
		if (LOBYTE(LOWORD(dwMemVal)) == 'R' && HIBYTE(LOWORD(dwMemVal)) == 'S' &&
			LOBYTE(HIWORD(dwMemVal)) == 'D' && HIBYTE(HIWORD(dwMemVal)) == ' ') {
			dwMemVal = _inmdw((DWORD_PTR)dwAdr + 4);
			if (LOBYTE(LOWORD(dwMemVal)) == 'P' && HIBYTE(LOWORD(dwMemVal)) == 'T' &&
				LOBYTE(HIWORD(dwMemVal)) == 'R' && HIBYTE(HIWORD(dwMemVal)) == ' ') {

				// Citeste adresa fizica a tabelei RSDT
				dwMemVal = _inmdw((DWORD_PTR)dwAdr + 16);
				pbAdrRSDT = (PBYTE)(UINT_PTR)dwMemVal;
				dwLungRSDT = _inmdw((DWORD_PTR)pbAdrRSDT + 4);
				dwLungRSDT -= 36;					// scade lungimea antetului
				pbAdrRSDT += 36;					// adresa de inceput a zonei de pointeri

				// La adresa indicata de fiecare pointer din tabela, cauta semnatura "MCFG"
				for (i = 0, bMcfg = FALSE; i < (int)(dwLungRSDT / 4); i++) {
					// Citeste un pointer din tabela RSDT
					dwMemVal = _inmdw ((DWORD_PTR)pbAdrRSDT);
					pbAdrRSDT += 4;
					pbAdrTab = (PBYTE)(UINT_PTR)dwMemVal;		// adresa de inceput a unei tabele
					// Citeste semnatura
					dwMemVal = _inmdw((DWORD_PTR)pbAdrTab);
					if (LOBYTE(LOWORD(dwMemVal)) == 'M' && HIBYTE(LOWORD(dwMemVal)) == 'C' &&
						LOBYTE(HIWORD(dwMemVal)) == 'F' && HIBYTE(HIWORD(dwMemVal)) == 'G') {
						bMcfg = TRUE;
						break;
					}
				}
				if (bMcfg == FALSE) {				// nu s-a gasit tabela ACPI MCFG
					return 0;
				}

				pbAdrMCFG = pbAdrTab;				// memoreaza adresa de inceput a tabelei MCFG

				// Determina numarul de structuri cu informatii despre spatiul de configuratie extins
				// Citeste lungimea tabelei MCFG
				dwMemVal = _inmdw((DWORD_PTR)pbAdrTab + 4);
				dwMemVal -= 44;						// scade lungimea antetului
				j = (int)(dwMemVal / 16);			// numarul de structuri
				pbAdrTab += 44;						// adresa de inceput a primei structuri

				// Citeste adresa de baza din structura 0 a tabelei MCFG
				dwAdrExtLo = _inmdw((DWORD_PTR)pbAdrTab);
				dwAdrExtHi = _inmdw((DWORD_PTR)pbAdrTab + 4);
				qwAdrExt = ((DWORD64)dwAdrExtHi << 32) | dwAdrExtLo;
			}
			else return 1;							// nu s-a gasit structura RSDP
		}
	}
	return (qwAdrExt);
}

